/*
 * Copyright 2007, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id$
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/serial_reg.h>
#include <linux/interrupt.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <asm/time.h>

#include <typedefs.h>
#include <osl.h>
#include <bcmutils.h>
#include <bcmnvram.h>
#include <sbconfig.h>
#include <sbchipc.h>
#include <sbutils.h>
#include <hndmips.h>
#include <mipsinc.h>
#include <hndcpu.h>
#include <bcmdevs.h>

/* Global SB handle */
extern void *bcm947xx_sbh;
extern spinlock_t bcm947xx_sbh_lock;

/* Convenience */
#define sbh bcm947xx_sbh
#define sbh_lock bcm947xx_sbh_lock

extern int panic_timeout;
static int watchdog = 0;
static u8 *mcr = NULL;

#include <linux/netdevice.h>
#include <linux/netlink.h>
#include <linux/delay.h>
#include "wlioctl.h"
#include "sbchipc.h"

#define WLAN_N_IF_NAME          "eth2"
#define WLAN_G_IF_NAME          "eth3"
#define LED_BLINK_RATE_NORMAL   10
#define LED_BLINK_RATE_WPS      50
 
#define REG_EXT_IF_ADDR     0xBB000000
#define REG_EXT_IF_CONFIG   0x00000011
#define REG_EXT_IF_WAIT     0x0404142A

static void *led_sbh = NULL;

int wps_led_pattern = 0;
int wps_led_state = 1;
EXPORT_SYMBOL(wps_led_pattern);
EXPORT_SYMBOL(wps_led_state);

static int wps_led_state_old = 1;

static int led_init(void)
{
    chipcregs_t *regs;

    uint32 config_old, waitcount_old;
    uint32 config, waitcount;

    if (!(led_sbh = sb_kattach(NULL)))
        return -ENODEV;

    regs = (chipcregs_t *) sb_setcore(led_sbh, SB_CC, 0);

    W_REG(SB_CC, &regs->prog_config, REG_EXT_IF_CONFIG);
    W_REG(SB_CC, &regs->prog_waitcount, REG_EXT_IF_WAIT);

    return 0;
}

static void led_output(int pattern)
{
    uint16 *extIf = (uint16 *)REG_EXT_IF_ADDR; 
    *extIf = (uint16)~pattern;
}

static void wps_blink(void)
{
    static int interrupt_count = -1;

    interrupt_count++;
    if (interrupt_count == LED_BLINK_RATE_WPS * 2)
        interrupt_count = 0;
    
    if (interrupt_count == 0)
        led_output(0xFF);
    else
    if (interrupt_count == LED_BLINK_RATE_WPS)
        led_output(0);
}

static int normal_blink(void)
{
    static struct net_device *wlan_dev1 = NULL;
    static struct net_device *wlan_dev2 = NULL;
    struct net_device_stats *wlan_dev1_stats = NULL;
    struct net_device_stats *wlan_dev2_stats = NULL;
    static int interrupt_count = 0;
    static int pkt_cnt_old = 0;
    int    pkt_cnt = 0;

    interrupt_count++;
    if (interrupt_count == LED_BLINK_RATE_NORMAL/2)
        led_output(wps_led_pattern);
    if (interrupt_count < LED_BLINK_RATE_NORMAL)
        return 0;
 
    interrupt_count = 0;

    if (wlan_dev1_stats)
        pkt_cnt += wlan_dev1_stats->tx_packets + wlan_dev1_stats->rx_packets;
    else {
        wlan_dev1 = dev_get_by_name(WLAN_N_IF_NAME);
        if (wlan_dev1) {
            wlan_dev1_stats = wlan_dev1->get_stats(wlan_dev1);
            if (wlan_dev1_stats)
                pkt_cnt += wlan_dev1_stats->tx_packets + wlan_dev1_stats->rx_packets;
            dev_put(wlan_dev1);
        }
    }

    if (wlan_dev2_stats)
        pkt_cnt += wlan_dev2_stats->tx_packets + wlan_dev2_stats->rx_packets;
    else {
        wlan_dev2 = dev_get_by_name(WLAN_G_IF_NAME);
        if (wlan_dev2) {
            wlan_dev2_stats = wlan_dev2->get_stats(wlan_dev2);
            if (wlan_dev2_stats)
                pkt_cnt += wlan_dev2_stats->tx_packets + wlan_dev2_stats->rx_packets;
            dev_put(wlan_dev2);
        }
    }
    
    if (pkt_cnt != pkt_cnt_old) {
        pkt_cnt_old = pkt_cnt;
        led_output(0);
    }

    return 0;
}

void __init
bcm947xx_time_init(void)
{
	unsigned int hz;

	/*
	 * Use deterministic values for initial counter interrupt
	 * so that calibrate delay avoids encountering a counter wrap.
	 */
	write_c0_count(0);
	write_c0_compare(0xffff);

	if (!(hz = sb_cpu_clock(sbh)))
		hz = 100000000;

	printk("CPU: BCM%04x rev %d at %d MHz\n", sb_chip(sbh), sb_chiprev(sbh),
	       (hz + 500000) / 1000000);

	/* Set MIPS counter frequency for fixed_rate_gettimeoffset() */
	mips_counter_frequency = hz / 2;

	/* Set watchdog interval in ms */
	watchdog = simple_strtoul(nvram_safe_get("watchdog"), NULL, 0);

	/* Please set the watchdog to 3 sec if it is less than 3 but not equal to 0 */
	if (watchdog > 0) {
		if (watchdog < 3000)
			watchdog = 3000;
	}

	/* Set panic timeout in seconds */
	panic_timeout = watchdog / 1000;

    led_init();
}

#ifdef CONFIG_HND_BMIPS3300_PROF
extern bool hndprofiling;
#ifdef CONFIG_MIPS64
typedef u_int64_t sbprof_pc;
#else
typedef u_int32_t sbprof_pc;
#endif
extern void sbprof_cpu_intr(sbprof_pc restartpc);
#endif	/* CONFIG_HND_BMIPS3300_PROF */

static void
bcm947xx_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
#ifdef CONFIG_HND_BMIPS3300_PROF
	/*
	 * Are there any ExcCode or other mean(s) to determine what has caused
	 * the timer interrupt? For now simply stop the normal timer proc if
	 * count register is less than compare register.
	 */
	if (hndprofiling) {
		sbprof_cpu_intr(regs->cp0_epc +
		                ((regs->cp0_cause >> (CAUSEB_BD - 2)) & 4));
		if (read_c0_count() < read_c0_compare())
			return;
	}
#endif	/* CONFIG_HND_BMIPS3300_PROF */

	/* Generic MIPS timer code */
	timer_interrupt(irq, dev_id, regs);

	/* Set the watchdog timer to reset after the specified number of ms */
	if (watchdog > 0) {
		if (sb_chip(sbh) == BCM5354_CHIP_ID)
			sb_watchdog(sbh, WATCHDOG_CLOCK_5354 / 1000 * watchdog);
		else
			sb_watchdog(sbh, WATCHDOG_CLOCK / 1000 * watchdog);
	}

#ifdef	CONFIG_HWSIM
	(*((int *)0xa0000f1c))++;
#else
	/* Blink one of the LEDs in the external UART */
	if (mcr && !(jiffies % (HZ/2)))
		writeb(readb(mcr) ^ UART_MCR_OUT2, mcr);
#endif

    /* Blink LED depending of WPS status */
    if (!wps_led_state)
    {
        if (wps_led_state_old != 0)
            led_output(0);
    }
    else
    if (wps_led_state == 1)
    {
        normal_blink();
    }
    else
    if (wps_led_state == 2)
    {
        wps_blink();
    }
    wps_led_state_old = wps_led_state;
}

static struct irqaction bcm947xx_timer_irqaction = {
	bcm947xx_timer_interrupt,
	SA_INTERRUPT,
	0,
	"timer",
	NULL,
	NULL
};

void __init
bcm947xx_timer_setup(struct irqaction *irq)
{
	/* Enable the timer interrupt */
	setup_irq(7, &bcm947xx_timer_irqaction);
}
